import copy
import os
import sys
from unittest import TestSuite, TestLoader

from HtmlTestRunner import HTMLTestRunner

from integration.partial.integration_test_utils import IntegrationTestUtils
from integration.test_suites.advisory_test_suite import BaseAdvisoryTestSuite


class FullTestAdvisory(BaseAdvisoryTestSuite.AdvisoryTestSuite):

    def set_expected_values(self):
        self.EXPECTED_EA_IMAGE_COUNT = 3
        self.EXPECTED_ETH_IMAGE_COUNT = 3
        self.EXPECTED_EA_SHAPE_COUNT = 6


    def setUp(self) -> None:

        self.set_expected_values()

        if BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH is None or not os.path.isdir(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH):
        # if True:
            FullTestAdvisory.write_temp_run_config_files()
            FullTestAdvisory.run_dependent_pipelines()
            FullTestAdvisory.run_advisory_pipeline()
        else:
            print(f"output in {BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH} already written, skipping rerun")


    @staticmethod
    def write_temp_run_config_files():
        default_config = IntegrationTestUtils.DEFAULT_CONFIG_FILE_PATH
        default_config_dict: dict = IntegrationTestUtils.load_json_file(default_config)
        run_dict: dict = copy.deepcopy(default_config_dict)
        run_dict['WorkspacePathout'] = BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH
        run_dict['WorkspacePath'] = BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH
        # we need to run the env suit pipeline as well
        run_dict['Environment']['WORK_PATH'] = BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH
        run_dict['Environment']['INPUT_PATH'] = BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH
        run_dict['Environment']['OUTPUT_PATH'] = BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH

        #make sure we activate server download
        run_dict['Survey']['SkipServerDownload'] = False

        # survey_cred_file: str = run_dict['Survey']['ServerCredentialsFile']
        # run_dict['Survey']['ServerCredentialsFile'] = survey_cred_file.replace("Cred-ODK-EIAR.json", "Cred-ODK-EIAR-NO-WRSIS.json")
        # run_dict['Survey']['FormEdits'].pop('WRSIS', None)

        # may be reusing a non-timestamped output file during development, so allow extant TEST_OUT_PATH
        os.makedirs(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH, exist_ok = True)
        IntegrationTestUtils.TEMP_CONFIG_FILE_NAME = BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH + "temp_config.json"
        IntegrationTestUtils.write_json_file(run_dict, IntegrationTestUtils.TEMP_CONFIG_FILE_NAME)


    @staticmethod
    def run_dependent_pipelines():
        IntegrationTestUtils.run_external_pipeline("Deposition",
                                                   BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_START_DATE,
                                                   IntegrationTestUtils.EMAIL_CRED_PATH)
        IntegrationTestUtils.run_external_pipeline("Environment",
                                                   BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_START_DATE,
                                                   IntegrationTestUtils.EMAIL_CRED_PATH)

        previous_day_string: str = IntegrationTestUtils.get_day_before_as_string(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_START_DATE)
        IntegrationTestUtils.run_external_pipeline("Survey",
                                                   previous_day_string,
                                                   IntegrationTestUtils.EMAIL_CRED_PATH)
        pass

    @staticmethod
    def run_advisory_pipeline():
        component = 'Advisory'
        IntegrationTestUtils.run_unittest_pipeline(component, FullTestAdvisory.TEST_START_DATE)


if __name__ == '__main__':

    _parser = IntegrationTestUtils.build_arg_parser()

    _args = _parser.parse_args()
    _config_file: str = _args.config
    _outdir: str = _args.outdir
    _email_cred_path: str = _args.email_cred
    _test_report_dir: str = _args.test_report_dir
    _run_date_type: str = _args.run_date_type
    _custom_run_date: str = _args.custom_run_date

    IntegrationTestUtils.DEFAULT_CONFIG_FILE_PATH = _config_file

    nowstring: str = IntegrationTestUtils.get_now_string()
    # prefix: str = "temp_advisory_" + nowstring
    prefix: str = "temp_advisory"
    BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH = _outdir + prefix + os.sep
    IntegrationTestUtils.EMAIL_CRED_PATH = _email_cred_path

    BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_START_DATE = IntegrationTestUtils.generate_run_date(_run_date_type, _custom_run_date)

    BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_JOB_DIR = "ADVISORY_" + BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_START_DATE

    #  Now set the sys.argv to the unittest_args (leaving sys.argv[0] alone)
    sys.argv[1:] = _args.unittest_args
    tests: TestSuite = TestLoader().loadTestsFromTestCase(FullTestAdvisory)

    if _test_report_dir is None:
        _test_report_dir = BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_JOB_DIR

    runner = HTMLTestRunner(output=_test_report_dir, combine_reports = True)
    runner.run(tests)
