import copy
import os
import sys
from unittest import TestSuite, TestLoader

from HtmlTestRunner import HTMLTestRunner

from integration.partial.integration_test_utils import IntegrationTestUtils
from integration.test_suites.env_suit_test_suite import BaseEnvSuitTestSuite


class FullTestEnvSuit(BaseEnvSuitTestSuite.EnvSuitTestSuite):

    def set_expected_values(self):
        self.EA_CSV_COUNT: int = 18
        self.ETH_CSV_COUNT: int = 18
        self.EA_PNG_COUNT: int = 6
        self.ETH_PNG_COUNT: int = 6

    def setUp(self) -> None:

        self.set_expected_values()

        if BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_OUT_PATH is None or not os.path.isdir(
                BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_OUT_PATH):
            FullTestEnvSuit.write_temp_run_config_file()
            FullTestEnvSuit.run_env_pipeline()
        else:
            print(f"output in {BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_OUT_PATH} already written, skipping rerun")


    @staticmethod
    def write_temp_run_config_file():
        default_config = IntegrationTestUtils.DEFAULT_CONFIG_FILE_PATH
        default_config_dict: dict = IntegrationTestUtils.load_json_file(default_config)
        run_dict: dict = copy.deepcopy(default_config_dict)
        run_dict['WorkspacePathout'] = FullTestEnvSuit.TEST_OUT_PATH
        run_dict['WorkspacePath'] = FullTestEnvSuit.TEST_OUT_PATH
        run_dict['Environment']['WORK_PATH'] = FullTestEnvSuit.TEST_OUT_PATH
        run_dict['Environment']['INPUT_PATH'] = FullTestEnvSuit.TEST_OUT_PATH
        run_dict['Environment']['OUTPUT_PATH'] = FullTestEnvSuit.TEST_OUT_PATH
        run_dict['Environment']['EWS-Plotting']['Ethiopia']['FilterForCountry'] = "True"
        run_dict['Environment']['EWS-Plotting']['Kenya']['FilterForCountry'] = "True"

        os.makedirs(BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_OUT_PATH)
        IntegrationTestUtils.TEMP_CONFIG_FILE_NAME = BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_OUT_PATH + "temp_config.json"
        IntegrationTestUtils.write_json_file(run_dict, IntegrationTestUtils.TEMP_CONFIG_FILE_NAME)


    @staticmethod
    def run_env_pipeline():
        component = 'Environment'
        IntegrationTestUtils.run_external_pipeline(component,
                                                   FullTestEnvSuit.TEST_START_DATE,
                                                   IntegrationTestUtils.EMAIL_CRED_PATH)


if __name__ == '__main__':

    _parser = IntegrationTestUtils.build_arg_parser()

    _args = _parser.parse_args()
    _config_file: str = _args.config
    _outdir: str = _args.outdir
    _email_cred_path: str = _args.email_cred
    _test_report_dir: str = _args.test_report_dir
    _run_date_type: str = _args.run_date_type
    _custom_run_date: str = _args.custom_run_date

    IntegrationTestUtils.DEFAULT_CONFIG_FILE_PATH = _config_file

    nowstring: str = IntegrationTestUtils.get_now_string()
    prefix: str = "temp_env_" + nowstring
    # prefix: str = "temp_env"
    BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_OUT_PATH = _outdir + prefix + os.sep
    IntegrationTestUtils.EMAIL_CRED_PATH = _email_cred_path

    BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_START_DATE = IntegrationTestUtils.generate_run_date(_run_date_type,
                                                                                                   _custom_run_date)
    BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_JOB_DIR = "ENVIRONMENT_2.0_" + FullTestEnvSuit.TEST_START_DATE

    #  Now set the sys.argv to the unittest_args (leaving sys.argv[0] alone)
    sys.argv[1:] = _args.unittest_args
    tests: TestSuite = TestLoader().loadTestsFromTestCase(FullTestEnvSuit)

    if _test_report_dir is None:
        _test_report_dir = BaseEnvSuitTestSuite.EnvSuitTestSuite.TEST_JOB_DIR

    runner = HTMLTestRunner(output=_test_report_dir, combine_reports = True)
    runner.run(tests)
