import copy
import os
import unittest

from integration.partial.integration_test_utils import IntegrationTestUtils
from integration.test_suites.survey_test_suite import BaseSurveyTestSuite


class TestSurvey(BaseSurveyTestSuite.SurveyTestSuite):

    def set_expected_values(self):
        BaseSurveyTestSuite.SurveyTestSuite.TEST_START_DATE = '20221001'
        BaseSurveyTestSuite.SurveyTestSuite.TEST_JOB_DIR = "SURVEYDATA_" + BaseSurveyTestSuite.SurveyTestSuite.TEST_START_DATE


    def setUp(self) -> None:

        self.set_expected_values()

        if BaseSurveyTestSuite.SurveyTestSuite.TEST_OUT_PATH is None or not os.path.isdir(
                BaseSurveyTestSuite.SurveyTestSuite.TEST_OUT_PATH):
            TestSurvey.write_temp_run_config_file()
            TestSurvey.unpack_dependencies()
            TestSurvey.run_survey_pipeline()
        else:
            print(f"output in {BaseSurveyTestSuite.SurveyTestSuite.TEST_OUT_PATH} already written, skipping rerun")


    @staticmethod
    def write_temp_run_config_file():
        nowstring: str = IntegrationTestUtils.get_now_string()
        prefix: str = "temp_survey_" + nowstring
        # prefix: str = "temp_survey"

        default_config = IntegrationTestUtils.DEFAULT_CONFIG_FILE_PATH
        default_config_dict: dict = IntegrationTestUtils.load_json_file(default_config)
        run_dict: dict = copy.deepcopy(default_config_dict)
        # TestSurvey.TEST_OUT_PATH = run_dict['WorkspacePathout'] + prefix + os.sep
        BaseSurveyTestSuite.SurveyTestSuite.TEST_OUT_PATH = IntegrationTestUtils.TEST_WORKSPACE_PATH + prefix + os.sep
        run_dict['WorkspacePathout'] = TestSurvey.TEST_OUT_PATH
        run_dict['WorkspacePath'] = TestSurvey.TEST_OUT_PATH
        run_dict['Survey']['SkipServerDownload'] = True
        run_dict['ServerName'] = ''  # nothing, as local machine

        IntegrationTestUtils.write_json_file(run_dict, IntegrationTestUtils.TEMP_CONFIG_FILE_NAME)

    @staticmethod
    def unpack_dependencies():
        IntegrationTestUtils.unpack_zip(IntegrationTestUtils.EXAMPLE_SURVEY_FILE_PATH, TestSurvey.TEST_OUT_PATH)

    @staticmethod
    def run_survey_pipeline():
        component = 'Survey'
        IntegrationTestUtils.run_unittest_pipeline(component, TestSurvey.TEST_START_DATE)


if __name__ == '__main__':
    unittest.main()
