import abc
import glob
import os
import unittest

from integration.partial.integration_test_utils import IntegrationTestUtils


class BaseAdvisoryTestSuite:

    class AdvisoryTestSuite(unittest.TestCase):

        TEST_OUT_PATH: str = None
        TEST_START_DATE: str = None
        TEST_JOB_DIR: str = None

        @abc.abstractmethod
        def set_expected_values(self):
            self.EXPECTED_EA_IMAGE_COUNT = 0
            self.EXPECTED_ETH_IMAGE_COUNT = 0
            self.EXPECTED_EA_SHAPE_COUNT = 0


        def test_standard_run_input_status_success(self):
            status_file_path = os.path.join(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH, BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_JOB_DIR, "STATUS_SUCCESS")
            success_file_exists: bool = os.path.isfile(status_file_path)
            self.assertTrue(success_file_exists)


        def test_standard_run_input_all_docs_produced(self):
            east_africa_image_wildcard = os.path.join(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH,
                                                  BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_JOB_DIR,
                                                  "tight-layout",
                                                  "wheat_rust_advisory_template_EastAfrica_*.docx")
            ethiopia_image_wildcard = os.path.join(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH,
                                               BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_JOB_DIR,
                                               "tight-layout",
                                               "wheat_rust_advisory_template_Ethiopia_*.docx")

            ea_file_exists: bool = len(glob.glob(east_africa_image_wildcard)) == 1
            eth_file_exists: bool = len(glob.glob(ethiopia_image_wildcard)) == 1
            self.assertTrue(ea_file_exists)
            self.assertTrue(eth_file_exists)


        def test_standard_run_input_images_produced(self):
            east_africa_image_wildcard = os.path.join(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH,
                                                      BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_JOB_DIR,
                                                      "images",
                                                      "*eastafrica*.png")
            ethiopia_image_wildcard = os.path.join(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH,
                                                   BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_JOB_DIR,
                                                   "images",
                                                   "*ethiopia*.png")

            africa_image_count: int = len(glob.glob(east_africa_image_wildcard))
            ethiopia_image_count: int = len(glob.glob(ethiopia_image_wildcard))
            self.assertEqual(self.EXPECTED_EA_IMAGE_COUNT, africa_image_count)
            self.assertEqual(self.EXPECTED_ETH_IMAGE_COUNT, ethiopia_image_count)


        def test_standard_run_input_shapefiles_produced(self):
            east_africa_image_wildcard = os.path.join(BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_OUT_PATH,
                                                      BaseAdvisoryTestSuite.AdvisoryTestSuite.TEST_JOB_DIR,
                                                      "images",
                                                      "*eastafrica*.shp")

            africa_shape_count: int = len(glob.glob(east_africa_image_wildcard))
            self.assertEqual(self.EXPECTED_EA_SHAPE_COUNT, africa_shape_count)